#ifdef GL_FRAGMENT_PRECISION_HIGH
precision highp float;
#else
precision mediump float;
#endif

uniform sampler2D uRainSampler;
uniform sampler2D uBackgroundSampler;
uniform sampler2D uForegroundSampler;

uniform vec2 uOOSurfaceSize;

uniform vec2 uRainTexCoordScale;
uniform vec2 uRainTexCoordOffset;

#ifdef SCROLL_RAIN
uniform vec2 uRainScrollOffset;
#endif

#ifdef USE_TINT
uniform vec4 uTint;
#endif

varying vec2 vTexCoord;

const float kMinRefraction = 128.0;
const float kMaxRefraction = 512.0;

const float kRaindropBrightness = 1.04;

const vec3 kLuminanceCoefficients = vec3(0.2126, 0.7152, 0.0722);

vec3 alphaBlend(in vec3 bg, in vec3 fg, float alpha) {
    return bg + alpha * (fg - bg);
}

void main() {
#ifdef SCROLLABLE
	// Get texture coordinate that isn't affected by the scrolling effect
	vec2 rainTexCoord = gl_FragCoord.xy * uOOSurfaceSize;
	rainTexCoord.y = 1.0 - rainTexCoord.y;
#else
	vec2 rainTexCoord = vTexCoord;
#endif

	rainTexCoord = (rainTexCoord * uRainTexCoordScale) + uRainTexCoordOffset;

	// Sample the rain texture
	vec4 rainSample = texture2D(uRainSampler, rainTexCoord);

	// Un-premultiply alpha
	rainSample.rgb /= rainSample.a;

	// Tweak alpha
	rainSample.a = clamp(rainSample.a * 6.0 - 3.0, 0.0, 1.0);

	// Raindrop reflection & refraction
	vec2 reflectionUV = (rainSample.gr - 0.5) * 2.0; // [0, 1] -> [-1, 1]
	vec2 refractionUV = vTexCoord + (uOOSurfaceSize * reflectionUV * mix(kMinRefraction, kMaxRefraction, rainSample.b));

	// Sample the downscaled background with the reflected and refracted UV coordinates
	vec3 raindropColor = texture2D(uForegroundSampler, refractionUV).rgb;

	// Brighten
	raindropColor *= kRaindropBrightness;

#ifdef USE_TINT
	// Add tint
	float d = dot(kLuminanceCoefficients, raindropColor);
	raindropColor.rgb += uTint.rgb * d;
#endif

	// Sample background
	vec3 backgroundColor = texture2D(uBackgroundSampler, vTexCoord).rgb;

	gl_FragColor = vec4(alphaBlend(backgroundColor, raindropColor, rainSample.a), 1.0);
//	gl_FragColor = vec4(mix(backgroundColor, raindropColor, rainSample.a), 1.0);
}
