precision mediump float;

uniform float uControlPoints[4];

varying vec2 vTexCoord;

// Must match value in simulation code
const float kMaxHeight = 0.1;

// Vignette constants
const float kVigRadius = 0.75;
const float kVigSoftness = 0.2;
const float kVigColor = 0.7;

float bezier(float t) {   
	float t_sq    = t*t;
	float t_cu    = t_sq*t;
	float omt     = 1.0 - t;
	float omt_sq  = omt*omt;
	float omt_cu  = omt_sq*omt;
	float t1      = omt_cu        * uControlPoints[0];
	float t2      = 3.0*omt_sq*t  * uControlPoints[1];
	float t3      = 3.0*omt*t_sq  * uControlPoints[2];
	float t4      = t_cu          * uControlPoints[3];
	return t1+t2+t3+t4;
}

void main() {
	// Bezier curve
	float curve = bezier(vTexCoord.x) * kMaxHeight;
	float fringe = smoothstep(curve, curve - 0.02, vTexCoord.y);
	
	// Vignette
	vec2 centerVec = vTexCoord - vec2(0.5);
	float distCenter = length(centerVec);
	float vignette = smoothstep(kVigRadius, kVigRadius - kVigSoftness, distCenter);
	
	gl_FragColor = vec4(1.0, 1.0, 1.0, mix(kVigColor, fringe, vignette));
}
